==> o365-3.0.2.tar.gz: o365/modules/o365_outlook_calendar/src/Plugin/Block/CalendarBlock.php <==
<?php

namespace Drupal\o365_outlook_calendar\Plugin\Block;

use Drupal\Core\Datetime\DateFormatter;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Url;
use Drupal\o365\Block\O365UncachedBlockBase;
use Drupal\o365\HelperService;
use Drupal\o365\PersonaRenderService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\o365\GraphService;

/**
 * Provides a 'Upcoming Appointments' block.
 *
 * @Block(
 *   id = "o365_outlook_calendar",
 *   admin_label = @Translation("Office 365 - Upcoming Appointments"),
 *   category = @Translation("Office 365")
 * )
 */
class CalendarBlock extends O365UncachedBlockBase implements ContainerFactoryPluginInterface {

  /**
   * Drupal\o365\GraphService definition.
   *
   * @var \Drupal\o365\GraphService
   */
  protected $o365Graph;

  /**
   * Drupal\o365\HelperService definition.
   *
   * @var \Drupal\o365\HelperService
   */
  protected $helperService;

  /**
   * Drupal\Core\Datetime\DateFormatter definition.
   *
   * @var \Drupal\Core\Datetime\DateFormatter
   */
  protected $dateFormatter;

  /**
   * The persona render service.
   *
   * @var \Drupal\o365\PersonaRenderService
   */
  protected $personaRenderService;

  /**
   * Constructs a new CalendarBlock object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param string $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\o365\GraphService $o365_graph
   *   The GraphService definition.
   * @param \Drupal\o365\HelperService $helperService
   *   The HelperService definition.
   * @param \Drupal\Core\Datetime\DateFormatter $dateFormatter
   *   The DateFormatter definition.
   * @param \Drupal\o365\PersonaRenderService $personaRenderService
   *   The persona render service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, GraphService $o365_graph, HelperService $helperService, DateFormatter $dateFormatter, PersonaRenderService $personaRenderService) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->o365Graph = $o365_graph;
    $this->personaRenderService = $personaRenderService;
    $this->helperService = $helperService;
    $this->dateFormatter = $dateFormatter;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static($configuration, $plugin_id, $plugin_definition, $container->get('o365.graph'), $container->get('o365.helpers'), $container->get('date.formatter'), $container->get('o365.profile_render'));
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);
    $config = $this->getConfiguration();

    $form['amount'] = [
      '#type' => 'number',
      '#title' => $this->t('Amount of events to show.'),
      '#default_value' => $config['amount'] ?? 5,
      '#min' => 1,
      '#max' => 999,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $this->configuration['amount'] = $values['amount'];
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Core\TempStore\TempStoreException
   * @throws \League\OAuth2\Client\Provider\Exception\IdentityProviderException
   * @throws \Exception
   */
  public function build() {
    $config = $this->getConfiguration();
    $amount = $config['amount'] ?? 5;
    $build = [];
    $fromDateTimestamp = date('U');
    $toDateTimestamp = strtotime('now + 30 days');
    $selectFromDate = $this->helperService->createIsoDate($fromDateTimestamp);
    $selectEndDate = $this->helperService->createIsoDate($toDateTimestamp);
    $eventData = $this->o365Graph->getGraphData('/me/calendarview?$top=' . $amount . '&$orderby=start/dateTime&$select=subject,start,end,attendees,isOnlineMeeting,onlineMeeting,webLink&startdatetime=' . $selectFromDate . '&enddatetime=' . $selectEndDate);

    $items = [];
    if (isset($eventData['value']) && !empty($eventData['value'])) {
      $webLinkText = new TranslatableMarkup('This link opens in a new window');
      $urlOptions = [
        'attributes' => [
          'target' => '_blank',
          'title' => $webLinkText,
        ],
      ];

      foreach ($eventData['value'] as $event) {
        $eventUrl = Url::fromUri($event['webLink'], $urlOptions);
        $onlineMeeting = FALSE;
        if ($event["isOnlineMeeting"]) {
          $onlineMeeting = $event['onlineMeeting']['joinUrl'];
        }

        $webLink = Link::fromTextAndUrl($webLinkText, $eventUrl);
        $fromTs = $this->helperService->getTsFromDate($event['start']['dateTime'], $event['start']['timeZone']);
        $endTs = $this->helperService->getTsFromDate($event['end']['dateTime'], $event['end']['timeZone']);

        // Get the attendees for this event.
        $attendees = [];
        $this->getAttendees($event['attendees'], $attendees);

        $items[] = [
          '#theme' => 'o365_calendar',
          '#subject' => $event['subject'],
          '#fromTime' => $this->dateFormatter->format($fromTs, 'o365_outlook_calendar_time'),
          '#endTime' => $this->dateFormatter->format($endTs, 'o365_outlook_calendar_time'),
          '#fromDate' => $this->dateFormatter->format($fromTs, 'o365_outlook_calendar_day'),
          '#endDate' => $this->dateFormatter->format($endTs, 'o365_outlook_calendar_day'),
          '#attendees' => $attendees['attendees'] ?? FALSE,
          '#remainingAttendees' => $attendees['remaining'] ?? FALSE,
          '#webLink' => $webLink,
          '#onlineMeeting' => $onlineMeeting,
          '#onlineMeeting_class' => 'button--primary button',
        ];
      }
    }

    if (!empty($items)) {
      $build['content'] = [
        '#theme' => 'o365_calendar_list',
        '#list' => [
          '#theme' => 'item_list',
          '#items' => $items,
          '#attributes' => [
            'class' => ['o365-outlook-calendar-items'],
          ],
          '#wrapper_attributes' => [
            'class' => [
              'card__body',
            ],
          ],
        ],
        '#cache' => [
          'contexts' => [
            'user',
          ],
        ],
        '#attached' => [
          'library' => [
            'o365/persona',
            'o365_outlook_calendar/o365_outlook_calendar',
          ],
        ],
      ];

      return $build;
    }
  }

  /**
   * Get the attendees for a event.
   *
   * @param array $attendees
   *   The attendees list.
   * @param array|bool $result
   *   The result.
   */
  private function getAttendees(array $attendees, &$result) {
    if (!empty($attendees)) {
      $result['attendees'] = [];
      $result['remaining'] = FALSE;
      // Determine remaining, we only show 3.
      $count = count($attendees);
      if ($count > 3) {
        $result['remaining'] = $count - 3;
      }

      // Generate the initials and color per user.
      for ($count = 0; $count < 3; $count++) {
        if (isset($attendees[$count])) {
          $attendee = $attendees[$count];

          $value = [
            'bg' => $this->personaRenderService->getRandomPersonaColor(),
          ];

          $initials = explode(' ', $attendee['emailAddress']['name']);
          if (count($initials) > 1) {
            $first = reset($initials);
            $last = end($initials);

            $value['initials'] = strtoupper($first[0] . $last[0]);
          }
          else {
            $value['initials'] = strtoupper($initials[0][0]);
          }

          $result['attendees'][] = $value;
        }
      }
    }
    else {
      $result = FALSE;
    }
  }

}

==> o365-3.0.3.tar.gz: o365/modules/o365_outlook_calendar/src/Plugin/Block/CalendarBlock.php <==
<?php

namespace Drupal\o365_outlook_calendar\Plugin\Block;

use Drupal\Core\Datetime\DateFormatter;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Url;
use Drupal\o365\Block\O365UncachedBlockBase;
use Drupal\o365\HelperService;
use Drupal\o365\PersonaRenderService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\o365\GraphService;

/**
 * Provides a 'Upcoming Appointments' block.
 *
 * @Block(
 *   id = "o365_outlook_calendar",
 *   admin_label = @Translation("Upcoming Appointments"),
 *   category = @Translation("Office 365")
 * )
 */
class CalendarBlock extends O365UncachedBlockBase implements ContainerFactoryPluginInterface {

  /**
   * Drupal\o365\GraphService definition.
   *
   * @var \Drupal\o365\GraphService
   */
  protected $o365Graph;

  /**
   * Drupal\o365\HelperService definition.
   *
   * @var \Drupal\o365\HelperService
   */
  protected $helperService;

  /**
   * Drupal\Core\Datetime\DateFormatter definition.
   *
   * @var \Drupal\Core\Datetime\DateFormatter
   */
  protected $dateFormatter;

  /**
   * The persona render service.
   *
   * @var \Drupal\o365\PersonaRenderService
   */
  protected $personaRenderService;

  /**
   * Constructs a new CalendarBlock object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param string $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\o365\GraphService $o365_graph
   *   The GraphService definition.
   * @param \Drupal\o365\HelperService $helperService
   *   The HelperService definition.
   * @param \Drupal\Core\Datetime\DateFormatter $dateFormatter
   *   The DateFormatter definition.
   * @param \Drupal\o365\PersonaRenderService $personaRenderService
   *   The persona render service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, GraphService $o365_graph, HelperService $helperService, DateFormatter $dateFormatter, PersonaRenderService $personaRenderService) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->o365Graph = $o365_graph;
    $this->personaRenderService = $personaRenderService;
    $this->helperService = $helperService;
    $this->dateFormatter = $dateFormatter;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static($configuration, $plugin_id, $plugin_definition, $container->get('o365.graph'), $container->get('o365.helpers'), $container->get('date.formatter'), $container->get('o365.profile_render'));
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);
    $config = $this->getConfiguration();

    $form['amount'] = [
      '#type' => 'number',
      '#title' => $this->t('Amount of events to show.'),
      '#default_value' => $config['amount'] ?? 5,
      '#min' => 1,
      '#max' => 999,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $this->configuration['amount'] = $values['amount'];
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Core\TempStore\TempStoreException
   * @throws \League\OAuth2\Client\Provider\Exception\IdentityProviderException
   * @throws \Exception
   */
  public function build() {
    $config = $this->getConfiguration();
    $amount = $config['amount'] ?? 5;
    $build = [];
    $fromDateTimestamp = date('U');
    $toDateTimestamp = strtotime('now + 30 days');
    $selectFromDate = $this->helperService->createIsoDate($fromDateTimestamp);
    $selectEndDate = $this->helperService->createIsoDate($toDateTimestamp);
    $eventData = $this->o365Graph->getGraphData('/me/calendarview?$top=' . $amount . '&$orderby=start/dateTime&$select=subject,start,end,attendees,isOnlineMeeting,onlineMeeting,webLink&startdatetime=' . $selectFromDate . '&enddatetime=' . $selectEndDate);

    $items = [];
    if (isset($eventData['value']) && !empty($eventData['value'])) {
      $webLinkText = new TranslatableMarkup('This link opens in a new window');
      $urlOptions = [
        'attributes' => [
          'target' => '_blank',
          'title' => $webLinkText,
        ],
      ];

      foreach ($eventData['value'] as $event) {
        $eventUrl = Url::fromUri($event['webLink'], $urlOptions);
        $onlineMeeting = FALSE;
        if ($event["isOnlineMeeting"]) {
          $onlineMeeting = $event['onlineMeeting']['joinUrl'];
        }

        $webLink = Link::fromTextAndUrl($webLinkText, $eventUrl);
        $fromTs = $this->helperService->getTsFromDate($event['start']['dateTime'], $event['start']['timeZone']);
        $endTs = $this->helperService->getTsFromDate($event['end']['dateTime'], $event['end']['timeZone']);

        // Get the attendees for this event.
        $attendees = [];
        $this->getAttendees($event['attendees'], $attendees);

        $items[] = [
          '#theme' => 'o365_calendar',
          '#subject' => $event['subject'],
          '#fromTime' => $this->dateFormatter->format($fromTs, 'o365_outlook_calendar_time'),
          '#endTime' => $this->dateFormatter->format($endTs, 'o365_outlook_calendar_time'),
          '#fromDate' => $this->dateFormatter->format($fromTs, 'o365_outlook_calendar_day'),
          '#endDate' => $this->dateFormatter->format($endTs, 'o365_outlook_calendar_day'),
          '#attendees' => $attendees['attendees'] ?? FALSE,
          '#remainingAttendees' => $attendees['remaining'] ?? FALSE,
          '#webLink' => $webLink,
          '#onlineMeeting' => $onlineMeeting,
          '#onlineMeeting_class' => 'button--primary button',
        ];
      }
    }

    if (!empty($items)) {
      $build['content'] = [
        '#theme' => 'o365_calendar_list',
        '#list' => [
          '#theme' => 'item_list',
          '#items' => $items,
          '#attributes' => [
            'class' => ['o365-outlook-calendar-items'],
          ],
          '#wrapper_attributes' => [
            'class' => [
              'card__body',
            ],
          ],
        ],
        '#cache' => [
          'contexts' => [
            'user',
          ],
        ],
        '#attached' => [
          'library' => [
            'o365/persona',
            'o365_outlook_calendar/o365_outlook_calendar',
          ],
        ],
      ];

      return $build;
    }
  }

  /**
   * Get the attendees for a event.
   *
   * @param array $attendees
   *   The attendees list.
   * @param array|bool $result
   *   The result.
   */
  private function getAttendees(array $attendees, &$result) {
    if (!empty($attendees)) {
      $result['attendees'] = [];
      $result['remaining'] = FALSE;
      // Determine remaining, we only show 3.
      $count = count($attendees);
      if ($count > 3) {
        $result['remaining'] = $count - 3;
      }

      // Generate the initials and color per user.
      for ($count = 0; $count < 3; $count++) {
        if (isset($attendees[$count])) {
          $attendee = $attendees[$count];

          $value = [
            'bg' => $this->personaRenderService->getRandomPersonaColor(),
          ];

          $initials = explode(' ', $attendee['emailAddress']['name']);
          if (count($initials) > 1) {
            $first = reset($initials);
            $last = end($initials);

            $value['initials'] = strtoupper($first[0] . $last[0]);
          }
          else {
            $value['initials'] = strtoupper($initials[0][0]);
          }

          $result['attendees'][] = $value;
        }
      }
    }
    else {
      $result = FALSE;
    }
  }

}
